package com.alibaba.cchannel.core;

import java.util.List;
import com.alibaba.cchannel.rpc.ServiceResponse;

/**
 * Created by Gary on 14-5-7.
 */
interface IChannelService {

    /**
     * 获取当前通道对应的唯一的设备标识（通常应用端用来保存，实现自定义的账号与DeviceID的映射关系。）
     */
    String getDeviceID();

    /**
     * 添加自定义标签（服务端后续可以通过这个Tag进行定向推送）
     * @tag 自定义标签的名称
     */
    boolean addTag(String tag);

    /**
     * 删除自定义标签（客户端将不再收到来自这个Tag的定向推送）
     * @tag 自定义标签的名称
     */
    boolean removeTag(String tag);

    /**
     * 将应用的内部账号与当前的通常绑定（绑定后，应用的服务端可以通过这个账号进行定点推送）
     * @param appID   应用的唯一标识
     * @param platformId   应用会话接入平台唯一标识
     * @param sid  应用的会话唯一标识
     * @param account 应用的账户标识（用来实现应用内账户与设备之间的关联，在推送时使用）
     */
    void bindAccountWithSID(int appID, int platformId, String sid, String account);

    /**
     * 将应用的内部账号与当前的通常绑定（绑定后，应用的服务端可以通过这个账号进行定点推送）
     * @param appID   应用的唯一标识
     * @param appKey  应用的身份密钥
     * @param account 应用的账户标识（用来实现应用内账户与设备之间的关联，在推送时使用）
     * @Deprecated
     */
    void bindAccount(int appID, String appKey, String account);

    /**
     * 将应用的内部账号与当前的通常解绑（解绑后，应用的服务端就不可以通过这个账号进行定点推送）
     */
    void unbindAccount(String sid, int platformId);
    /**
     * 将应用与推送通道取消关联（后续将无法收到服务端的消息）
     */
    void unregister();

    /**
     * 请求内部的资源服务
     * @param url   资源的唯一标识
     * @param requestBody  请求对应的数据（如：参数）
     */
    String innerRequest(String url, String requestBody);

    /**
     * 强制唤醒客户端长连接通道（如果当前通道已经打开，此方法会被忽略）
     */
    void wakeup();

    /**
     * 负责消息的统计消息回执
     * @param messageID   消息的唯一标识
     * @param type  回执的类型（3：已送到，4：已读，8：已删除）
     * @param persistence  是否要持久化
     */
    void reportMessage(long messageId, byte type, boolean persistence);

    /**
     * 设置免打扰的时间段
     * @param startH   开始的小时
     * @param startMS  开始的分钟
     * @param endH   结束的小时
     * @param endMS  结束的分钟
     */
    void setAcceptTime(int startH,int startMS, int endH, int endMS);

    /**
     * 返回保持长连接的主应用的包名
     */
    String getMainPackageName();

    /**
     * 返回已经关联上的用户应用
     */
    List<String> getAttachedApplications();

    /**
     * 返回SDK内核的版本号
     */
    byte getSDKVersion();

    /**
     * 重新启动服务
     */
    void restart();

    /**
     * 是否已经可以正常通信
     */
    boolean isReadyForCommunicate();

    /**
     * 远程服务调用
     * @param version          RPC协议版本号
     * @param platformId       平台唯一标识
     * @param sid              会话唯一标识
     * @param payload          请求体（资源、可变头、参数）
     */
    ServiceResponse request(byte version, int platformId, String sid, in byte[] payload);

    /**
     * 映射Master的应用信息（这是一个内部方法，仅由真正的Master调用才会生效）
     */
    void _mappingMaster(int appID, String account);

    /**
     * 重新连接
    */
    void reconnect(String sid);
}
